@@ -5,7 +5,7 @@ from arkitekt.actors.base import Actor
 from arkitekt.packers.structure import Structure
 from arkitekt.schema.node import Node
 from arkitekt.schema.enums import NodeType
-from arkitekt.schema.ports import ArgPort, DictArgPort, DictKwargPort, DictReturnPort, EnumArgPort, EnumKwargPort, IntArgPort, IntKwargPort, IntReturnPort, KwargPort, ListArgPort, ListKwargPort, ListReturnPort, ReturnPort, StringArgPort, StringKwargPort, StringReturnPort, StructureArgPort, StructureKwargPort, StructureReturnPort
+from arkitekt.schema.ports import ArgPort, BoolArgPort, BoolKwargPort, BoolReturnPort, DictArgPort, DictKwargPort, DictReturnPort, EnumArgPort, EnumKwargPort, IntArgPort, IntKwargPort, IntReturnPort, KwargPort, ListArgPort, ListKwargPort, ListReturnPort, ReturnPort, StringArgPort, StringKwargPort, StringReturnPort, StructureArgPort, StructureKwargPort, StructureReturnPort
 from arkitekt.actors.functional import FunctionalFuncActor, FunctionalGenActor, FunctionalThreadedFuncActor, FunctionalThreadedGenActor
 from arkitekt.packers.transpilers.registry import get_transpiler_registry
 
@@ -53,6 +53,8 @@ def convert_arg_to_argport(cls, **kwargs) -> ArgPort:
 
     if inspect.isclass(cls):
         # Generic Cases
+
+        if issubclass(cls, bool): return BoolArgPort.from_params(**kwargs) # catch bool is subclass of int
         if issubclass(cls, Enum): return EnumArgPort.from_params(options={key: value._value_ for key,value in cls.__members__.items()},**kwargs)
         if issubclass(cls, int): return IntArgPort.from_params(**kwargs)
         if issubclass(cls, str): return StringArgPort.from_params(**kwargs)
@@ -89,6 +91,8 @@ def convert_kwarg_to_kwargport(cls, **kwargs):
     
     if inspect.isclass(cls):
         # Generic Cases
+
+        if issubclass(cls, bool): return BoolKwargPort.from_params(**kwargs) # bool subclass of int therefore before
         if issubclass(cls, Enum): return EnumKwargPort.from_params(options={key: value._value_ for key,value in cls.__members__.items()},**kwargs)
         if issubclass(cls, int): return IntKwargPort.from_params(**kwargs)
         if issubclass(cls, str): return StringKwargPort.from_params(**kwargs)
@@ -120,6 +124,8 @@ def convert_return_to_returnport(cls, **kwargs):
     # Generic Cases
     if inspect.isclass(cls):
         # Generic Cases
+
+        if issubclass(cls, bool): return BoolReturnPort.from_params(**kwargs)
         if issubclass(cls, int): return IntReturnPort.from_params(**kwargs)
         if issubclass(cls, str): return StringReturnPort.from_params(**kwargs)
 
@@ -146,7 +152,9 @@ def define(function, widgets={}, allow_empty_doc=False) -> Node:
     """Define
 
     Define a functions in the context of arnheim and
-    return it as a Node.
+    return it as a Node. Attention this node is not yet 
+    hosted on Arkitekt (doesn't have an id). So make sure
+    to save this node before calling it anywhere
 
     Args:
         function (): The function you want to define
@@ -196,7 +204,7 @@ def define(function, widgets={}, allow_empty_doc=False) -> Node:
 
 
     name = docstring.short_description or function.__name__
-    interface = inflection.underscore(name) # convert this to camelcase
+    interface = inflection.underscore(function.__name__) # convert this to camelcase
     description = docstring.long_description or "No Description"
 
     doc_param_map = {param.arg_name: {
@@ -234,15 +242,16 @@ def define(function, widgets={}, allow_empty_doc=False) -> Node:
     })
 
 
-async def async_none(self, message):
+async def async_none(message):
     return None
 
 
-def actify(function_or_actor, bypass_shrink=False, bypass_expand=False, transpilers: Dict[str, Transpiler] = None, on_provide=None, on_unprovide=None, **params):
-    if isactor(function_or_actor): return function_or_actor
-    is_method = inspect.ismethod(function_or_actor)
 
-    actor_name = f"GeneratedActor{function_or_actor.__name__.capitalize()}"
+
+def actify(function_or_actor, bypass_shrink=False, bypass_expand=False, transpilers: Dict[str, Transpiler] = None, on_provide=None, on_unprovide=None, actor_name= None, **params):
+    if isactor(function_or_actor): return function_or_actor
+
+    actor_name = actor_name or f"GeneratedActor{function_or_actor.__name__.capitalize()}"
 
 
     is_coroutine = inspect.iscoroutinefunction(function_or_actor)
@@ -250,9 +259,10 @@ def actify(function_or_actor, bypass_shrink=False, bypass_expand=False, transpil
 
     is_generatorfunction = inspect.isgeneratorfunction(function_or_actor)
     is_function = inspect.isfunction(function_or_actor)
-    print(on_provide)
-    class_attributes = {
-        "assign": staticmethod(function_or_actor),
+    
+
+    actor_attributes = {
+        "assign": function_or_actor,
         "expand_inputs": not bypass_expand,
         "shrink_outputs":  not bypass_shrink,
         "transpilers": transpilers,
@@ -261,13 +271,13 @@ def actify(function_or_actor, bypass_shrink=False, bypass_expand=False, transpil
     }
 
     if is_coroutine:
-        return type(actor_name,(FunctionalFuncActor,), class_attributes)
+        return FunctionalFuncActor(**actor_attributes)
     elif is_asyncgen:
-        return  type(actor_name,(FunctionalGenActor,), class_attributes)
+        return  FunctionalGenActor(**actor_attributes)
     elif is_generatorfunction:
-        return type(actor_name, (FunctionalThreadedGenActor,),class_attributes)
+        return FunctionalThreadedGenActor(**actor_attributes)
     elif is_function:
-        return type(actor_name,(FunctionalThreadedFuncActor,), class_attributes)
+        return FunctionalThreadedFuncActor(**actor_attributes)
     else:
         raise NotImplementedError("No way of converting this to a function")
 
