@@ -32,15 +32,8 @@ logger = logging.getLogger(__name__)
 class AppAgent(Agent):
     ACTOR_PENDING_MESSAGE = "Actor is Pending"
 
-    def __init__(self, *args, strict=False, **kwargs) -> None:
+    def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
-
-        self.potentialNodes: List[Tuple[Node, Callable]] = []
-        self.potentialTemplates: List[Tuple[Node, Callable]] = []
-        self.approvedTemplates: List[Tuple[Template, Callable]] = []
-        self.approvedActors: Dict[str, Type[Actor]] = {}
-        self.strict = strict
-
         # Running Actors indexed by their ID
         self.runningActors: Dict[str, Actor] = {}
         self.runningTasks: Dict[str, asyncio.Task] = {}
@@ -50,20 +43,22 @@ class AppAgent(Agent):
 
     async def on_bounced_provide(self, message: BouncedProvideMessage):
         
-        if message.data.template in self.approvedActors:
-            if message.meta.reference in self.runningActors:
+        if message.data.template in self.templateActorsMap:
+            if message.meta.reference not in self.runningActors:
+                # Didn not exist before
+                actor = self.templateActorsMap[message.data.template] # creating out little Actor
+                self.runningActors[message.meta.reference] = actor
+                task = self.loop.create_task(actor.arun(message, self))
+                task.add_done_callback(self.on_task_done)
+                self.runningTasks[message.meta.reference] = task
+                
+            else:
                 if self.strict: raise AgentException("Already Running Provision Received Again. Right now causing Error. Might be omitted")
                 again_provided = ProvideTransitionMessage(data={
                     "message": "Provision was running on this Instance. Probably a freaking race condition",
                     "state": ProvideState.ACTIVE
                     }, meta={"extensions": message.meta.extensions, "reference": message.meta.reference})
                 await self.transport.forward(again_provided)
-            else:
-                actor = self.approvedActors[message.data.template](message, self.transport, monitor=self.monitor)
-                self.runningActors[message.meta.reference] = actor
-                task = self.loop.create_task(actor.arun())
-                task.add_done_callback(self.on_task_done)
-                self.runningTasks[message.meta.reference] = task
 
         else:
             raise AgentException("No approved actors for this template")
@@ -76,6 +71,7 @@ class AppAgent(Agent):
         logger.info(f"Cancelling {actor}")
         self.runningTasks[message.data.provision].cancel()
 
+
     async def on_bounced_assign(self, message: BouncedForwardedAssignMessage):
 
         if message.data.provision in self.runningActors:
@@ -193,102 +189,23 @@ class AppAgent(Agent):
             def wrapped_function(*args, **kwargs):
                 return function(*args, **kwargs)
 
+            
+            defined_actor = actify(function, on_provide=on_provide, on_unprovide=on_unprovide, **params)
+
             if len(args) == 0:
                 defined_node = define(function=function, widgets=widgets)
-                defined_actor = actify(function, on_provide=on_provide, on_unprovide=on_unprovide, **params)
-
-
-                self.potentialNodes.append((defined_node, defined_actor, params))
+                self.templatedNewNodes.append((defined_node, defined_actor, params))
 
             if len(args) == 1:
-                print("Hallo")
+                if isinstance(args[0], str):
+                    self.templatedUnqueriedNodes.append(({"q": args[0]}, defined_actor, params))
+
+                if isinstance(args[0], Node):
+                    self.templatedNodes.append((args[0], defined_actor, params))
+
                 # We are registering this as a template
-                
 
             return wrapped_function
 
     
         return real_decorator
-
-    
-   
-
-
-    async def providing_loop(self):
-        await self.transport.connect()
-
-        while True:
-            await asyncio.sleep(3)
-            print("RUnning")
-
-        if self.monitor: self.monitor.__exit__()
-        if self.panel: self.panel.end()
-
-
-
-    async def broadcast(self, message: MessageModel):
-
-        logger.info(f"Provider: [green] Received message {message}")
-        try:
-            if isinstance(message, BouncedProvideMessage):
-                logger.info("Received Provide Request")
-                assert message.data.template is not None, "Received Provision that had no Template???"
-                await self.handle_bounced_provide(message)
-
-            elif isinstance(message, BouncedUnprovideMessage):
-                logger.info("Received Unprovide Request")
-                assert message.data.provision is not None, "Received Unprovision that had no Provision???"
-                await self.handle_bounced_unprovide(message)
-
-            elif isinstance(message, BouncedForwardedAssignMessage):
-                logger.info("Received Assing Request")
-                assert message.data.provision is not None, "Received Assignment that had no Provision???"
-                await self.handle_bounced_assign(message)
-
-            elif isinstance(message, BouncedForwardedUnassignMessage):
-                logger.info("Received Unassign Request")
-                assert message.data.provision is not None, "Received Assignment that had no Provision???"
-                await self.handle_bounced_unassign(message)
-
-            else: 
-                logger.info(f"Provider: [red] Received unknown message {message}")
-        except Exception as e:
-            logger.exception(e)
-
-    async def aprovide(self):
-        # Connection Started
-        if not self.connected: 
-            await self.connect()
-
-        
-        if self.monitor: self.monitor.__enter__()
-        if self.panel: self.panel.start()
-
-        if self.potentialNodes:
-            for defined_node, defined_actor, params in self.potentialNodes:
-                # Defined Node are nodes that are not yet reflected on arkitekt (i.e they dont have an instance
-                # id so we are trying to send them to arkitekt)
-                try:
-                    arkitekt_node = await Node.asyncs.create(**defined_node.dict(as_input=True))
-                    self.potentialTemplates.append((arkitekt_node, defined_actor, params))   
-                except WardException as e:
-                    raise AgentException(f"Couldn't create Node for defintion {defined_node}") from e
-
-        if self.potentialTemplates:
-            # This is an arkitekt Node and we can generate potential Templates
-            for arkitekt_node, defined_actor, params in self.potentialTemplates:
-                try:
-                    params = await parse_params(params) # Parse the parameters for template creation
-                    arkitekt_template = await Template.asyncs.create(node=arkitekt_node, params=params)
-                    self.approvedTemplates.append((arkitekt_template, defined_actor, params))  
-                except WardException as e:
-                    raise AgentException(f"Couldn't approve template for node {arkitekt_node}") from e
-
-
-        if self.approvedTemplates:
-            for arkitekt_template, defined_actor, params in self.approvedTemplates:
-                self.approvedActors[arkitekt_template.id] = defined_actor
-                if self.panel: self.panel.add_to_actor_map(arkitekt_template, defined_actor) 
-        
-        
-        await self.providing_loop()
\ No newline at end of file
